-- MenuButton class as a minimum user interface to toggle the menu.
-- By default, the button is positioned to the bottom left corner of the display

import "Turbine.UI";

import "Equendil.UI.DragBox";

IconOnOff = { -- 32x32
	"Equendil/UI/images/onoff.tga",
	"Equendil/UI/images/onoff_highlighted.tga",
	"Equendil/UI/images/onoff_depressed.tga",
};

MenuButton = class( Turbine.UI.Window );
function MenuButton:Constructor( menu )
	Turbine.UI.Window.Constructor( self );
	
	self.Changed = Equendil.Utils.Event(); -- Event called without parameters if position or scale changes
	
	self.menu = menu;

	self.scale = 1;
	self.opacity = 0.5;
	
	self:SetZOrder( 100 );
	self:SetOpacity( self.opacity );
	self:SetSize( 32, 32 );
	self:SetStretchMode( 1 );
	self:SetPosition( 0, Turbine.UI.Display:GetHeight() - 32 ); -- we go to the bottom / left of the display by default
	self:SetText( "Bootstrap's menu button" );
	self:SetVisible( true );
	
	Equendil.UI.WindowManager:ManageWindow( self, false ); -- for display toggle handling, no closing on escape
	
	-- attach a drag bar to it for repositioning (CTRL+\)
	self.dragBox = Equendil.UI.DragBox( self );
	self.dragBox.Repositioned = function() self.Changed(); end
	Equendil.UI.WindowManager:ManageDragBar( self.dragBox ); -- the window manager handles will handle visilbity 
	self.menuButton = Equendil.UI.IconButton( IconOnOff, 32, 32 );
	self.menuButton:SetBlendMode( Turbine.UI.BlendMode.Normal );
	self.menuButton:SetParent( self );
	
	self.menuButton.Clicked = function() -- TODO: handle loss of focus of menu -> going invisible while in menuShowing = true state
		if not self.menu:IsVisible() then
			local left, top = 0, 0;
			if self:GetLeft() > Turbine.UI.Display.GetWidth() / 2 then
				left = self:GetLeft() - self.menu:GetWidth() + self:GetWidth(); 
			else
				left = self:GetLeft();
			end
			
			if self:GetTop() > Turbine.UI.Display.GetHeight() / 2 then
				top = self:GetTop(); -- menu will handle its actual position from its bottom
			else
				top = self:GetTop() + self:GetHeight();
			end
			self.menu:SetPosition( left, top );
		end
		self.menu:Show( not self.menu:IsVisible() );
	end
	
	self.actionSetOpacity = function( flag )
		if flag then
			Turbine.UI.Window.SetOpacity( self, 1.0 );
		else
			Turbine.UI.Window.SetOpacity( self, self.opacity );
		end
	end

	-- We go opaque either when the mouse is hovering the button or in repositioning mode
	self.menuButton.Hovering = self.actionSetOpacity;
	Equendil.UI.WindowManager.GUIRepositioning:Add(	self.actionSetOpacity );

	self.dragBox.Scroll = function( direction ) 
		if direction > 0 then
			self:SetScale( math.min( self.scale + 0.25, 1 ) );
		else
			self:SetScale( math.max( self.scale - 0.25, 0.5 ) );
		end
	end
end

function MenuButton:GetOpacity()
	return self.opacity;
end

function MenuButton:SetOpacity( opacity )
	self.opacity = opacity;
	Turbine.UI.Window.SetOpacity( self, self.opacity );
	self:SetVisible( opacity ~= 0 );
	self.Changed();
end

function MenuButton:GetScale()
	return self.scale;
end

function MenuButton:_DoLayout()
	self:SetSize( 32 * self.scale, 32 * self.scale );
end

function MenuButton:SetScale( scale )
	self.scale = scale;
	local width, height = self:GetSize();
	self:_DoLayout();
	self.Changed();
end

-- Stetch mode 1 causes GetSize() etc on Turbine.UI.Window to return unstretched values, so we overload them and return correct values
function MenuButton:GetSize()
	return self:GetWidth(), self:GetHeight();
end

function MenuButton:GetWidth()
	return Turbine.UI.Window.GetWidth( self ) * self.scale;
end

function MenuButton:GetHeight()
	return Turbine.UI.Window.GetHeight( self ) * self.scale;
end